#include <QtCore>
#include <QtGui>

#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_plot_panner.h>
#include <qwt_plot_magnifier.h>
#include <qwt_plot_layout.h>

#include "charts.h"

ChartDialog::ChartDialog(QExchange *exchangeModule, QWidget *parent) : QDialog(parent), module(exchangeModule)
{
  QFont smallerFont(QApplication::font());
  QMap<QString,QString> currencies = module->currencies();
  QMapIterator<QString,QString> currency(currencies);
  QProgressDialog progressDlg(parent);

  xData = yData = 0;
  zoomer = 0;
  setupUi(this);

  progressDlg.setMinimum(0);
  progressDlg.setMaximum(currencies.size());
  progressDlg.setCancelButton(0);
  progressDlg.setWindowTitle("Loading...");
  progressDlg.setLabelText("Loading historical data...");
  progressDlg.setWindowIcon(QIcon(":/images/chart"));
  progressDlg.setValue(0);
  progressDlg.setAutoClose(true);
  progressDlg.setModal(true);

  while (currency.hasNext())
  {
    if (module->hasData(currency.next().key()))
    {
      cbLeftCurrency->addItem(currency.value() + " (" + currency.key() + ")",
                              currency.key());
      cbRightCurrency->addItem(currency.value() + " (" + currency.key() + ")",
                               currency.key());
    }
    progressDlg.setValue(progressDlg.value()+1);
    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
  }

  cbLeftCurrency->setCurrentIndex(0);
  cbLeftCurrency->setCurrentIndex(1);

  connect(cbLeftCurrency,SIGNAL(currentIndexChanged(int)),this,SLOT(replot()));
  connect(cbRightCurrency,SIGNAL(currentIndexChanged(int)),this,SLOT(replot()));

  smallerFont.setPointSize(smallerFont.pointSize()-1);
  qpChart->setAxisScaleDraw(QwtPlot::xBottom,new ChartDateScale);
  qpChart->setAxisFont(QwtPlot::yLeft,smallerFont);
  qpChart->setAxisFont(QwtPlot::xBottom,smallerFont);

  qpChart->plotLayout()->setAlignCanvasToScales(true);

  QwtPlotGrid *grid = new QwtPlotGrid;
  grid->setMajPen(QPen(Qt::gray,0,Qt::DotLine));
  grid->attach(qpChart);

  QwtPlotPanner *panner = new QwtPlotPanner(qpChart->canvas());
  panner->setAxisEnabled(QwtPlot::yRight,false);
  panner->setAxisEnabled(QwtPlot::xTop,false);
  panner->setMouseButton(Qt::MidButton);

  move((QApplication::desktop()->screenGeometry().width() - width()) / 2,
       (QApplication::desktop()->screenGeometry().height() - height()) / 2);

  replot();
}

ChartDialog::~ChartDialog()
{
  qpChart->clear();
  delete xData;
  delete yData;
}

void ChartDialog::replot()
{
  int i;
  QwtPlotCurve *curve;
  QMap<int,qreal> history;

  qpChart->clear();
  delete xData;
  xData = 0;
  delete yData;
  yData = 0;
  delete zoomer;
  zoomer = 0;

  qpChart->setAxisAutoScale(QwtPlot::yLeft);
  qpChart->setAxisAutoScale(QwtPlot::xBottom);
  qpChart->setAxisTitle(QwtPlot::yLeft,cbRightCurrency->itemData(cbRightCurrency->currentIndex()).toString());
  qpChart->setAxisTitle(QwtPlot::xBottom,cbLeftCurrency->itemData(cbLeftCurrency->currentIndex()).toString());

  history = module->rateHistory(cbLeftCurrency->itemData(cbLeftCurrency->currentIndex()).toString(),
                                cbRightCurrency->itemData(cbRightCurrency->currentIndex()).toString());

  if (history.size())
  {
    QMapIterator<int,qreal> exchange(history);

    curve = new QwtPlotCurve;
    curve->setPen(QPen(Qt::red));
    xData = new double[history.size()];
    yData = new double[history.size()];

    i = 0;
    while (exchange.hasNext())
    {
      exchange.next();
      xData[i] = exchange.key();
      yData[i++] = exchange.value();
    }

    curve->setRawData(xData,yData,history.size());
    curve->attach(qpChart);

    zoomer = new ChartDateZoomer(qpChart->canvas());
    zoomer->setRubberBandPen(QPen(Qt::red,2,Qt::DotLine));
    zoomer->setTrackerPen(QPen(Qt::red));
    zoomer->setMousePattern(QwtEventPattern::MouseSelect2,Qt::RightButton,Qt::ControlModifier);
    zoomer->setMousePattern(QwtEventPattern::MouseSelect3,Qt::RightButton);

  }
  else
  {
    (new ChartNoPlot)->attach(qpChart);
    qpChart->replot();
  }
}

void ChartNoPlot::draw(QPainter *painter, const QwtScaleMap & /* xMap */,
                       const QwtScaleMap & /* yMap */, const QRect &canvasRect) const
{
  QFont biggerFont(QApplication::font());

  biggerFont.setPointSize(biggerFont.pointSize()+6);
  biggerFont.setBold(true);
  painter->setFont(biggerFont);
  painter->drawText(canvasRect,Qt::AlignHCenter | Qt::AlignVCenter,"No data available");
}
