#include <QtCore>
#include <QtGui>

#include "mainwindow.h"
#include "qexchange.h"
#include "preferencesdlg.h"
#include "charts.h"

MainWindow::MainWindow(QExchange *exchangeModule, QWidget *parent) : QMainWindow(parent)
{
  QDoubleValidator *validator;

  skipSignal = true;

  module = exchangeModule;

  setupUi(this);

  validator = new QDoubleValidator(leLeftValue);
  validator->setNotation(QDoubleValidator::StandardNotation);
  leLeftValue->setValidator(validator);
  validator = new QDoubleValidator(leRightValue);
  validator->setNotation(QDoubleValidator::StandardNotation);
  leRightValue->setValidator(validator);

  connect(actionAbout_Qt,SIGNAL(triggered()),qApp,SLOT(aboutQt()));
  connect(module,SIGNAL(changed()),this,SLOT(recalculate()));
  connect(actionUpdate,SIGNAL(triggered()),module,SLOT(update()));
  connect(module,SIGNAL(startDownload()),this,SLOT(downloadStarted()));
  connect(module,SIGNAL(downloadFailed()),this,SLOT(downloadFailed()));
  connect(module,SIGNAL(downloadCompleted()),this,SLOT(downloadCompleted()));
  connect(module,SIGNAL(storeInProgress(int,int,int)),this,SLOT(showStoreProgress(int,int,int)));
  connect(module,SIGNAL(downloadInProgress(int,int,int)),this,SLOT(showDownloadProgress(int,int,int)));

  skipSignal = false;

  if (module->hasData())
  {
    reload();

    leLeftValue->setFocus();
  }
  else
  {
    reloadAll();

    centralWidget()->setEnabled(false);
    actionChart->setEnabled(false);
  }

  setWindowTitle(QString(APPLICATION_NAME) + " - " + QString(APPLICATION_VERSION));
  lblExchangeDirection->setPixmap(QPixmap(":/images/to_right"));
  layout()->setSizeConstraint(QLayout::SetFixedSize);

  move((QApplication::desktop()->screenGeometry().width() - width()) / 2,
       (QApplication::desktop()->screenGeometry().height() - height()) / 2);
}

void MainWindow::on_leLeftValue_textChanged(const QString & /* text */)
{
  if (!skipSignal)
  {
    convertLeftToRight = true;
    recalculate();
  }
}

void MainWindow::on_leRightValue_textChanged(const QString & /* text */)
{
  if (!skipSignal)
  {
    convertLeftToRight = false;
    recalculate();
  }
}

void MainWindow::on_cbLeftCurrency_currentIndexChanged(int index)
{
  if (!skipSignal)
  {
    convertLeftToRight = false;
    module->setFromCurrency(cbRightCurrency->itemData(cbRightCurrency->currentIndex()).toString());
    module->setToCurrency(cbLeftCurrency->itemData(index).toString());
  }
}

void MainWindow::on_cbRightCurrency_currentIndexChanged(int index)
{
  if (!skipSignal)
  {
    convertLeftToRight = true;
    module->setFromCurrency(cbLeftCurrency->itemData(cbLeftCurrency->currentIndex()).toString());
    module->setToCurrency(cbRightCurrency->itemData(index).toString());
  }
}

void MainWindow::on_deExchangeDate_dateChanged(const QDate & /* date */)
{
  if (!skipSignal)
    recalculate();
}

void MainWindow::on_actionChart_triggered()
{
  ChartDialog dlg(module,this);

  dlg.exec();
}

void MainWindow::on_actionAbout_triggered()
{
  QMessageBox::about(this,QString("About ") + QString(APPLICATION_NAME),
                     QString("<b><font size=\"+2\">") + QString(APPLICATION_NAME) + " " +
                     QString(APPLICATION_VERSION) + "</font></b><br><br>" +
                     "Author: Antonio Di Monaco");
}

void MainWindow::on_actionPreferences_triggered()
{
  PreferencesDialog dlg(this);

  dlg.setSettings(module->settings());

  if (dlg.exec() == QDialog::Accepted)
    module->setNewSettings(dlg.settings());
}

void MainWindow::reloadAll()
{
  QMap<QString,QString> currencies;

  try
  {
    skipSignal = true;
    cbLeftCurrency->clear();
    cbRightCurrency->clear();

    currencies = module->currencies();

    QMapIterator<QString,QString> currency(currencies);

    cbLeftCurrency->addItem("Euro (EUR)","EUR");
    cbRightCurrency->addItem("Euro (EUR)","EUR");
    while (currency.hasNext())
    {
      currency.next();
      cbLeftCurrency->addItem(currency.value() + " (" + currency.key() + ")",
                              currency.key());
      cbRightCurrency->addItem(currency.value() + " (" + currency.key() + ")",
                               currency.key());
    }

    cbLeftCurrency->setCurrentIndex(0);
    cbRightCurrency->setCurrentIndex(1);

    skipSignal = false;
  }
  catch(const QString &msg)
  {
    QErrorMessage(this).showMessage(msg);
  }
}

void MainWindow::reload()
{
  QMap<QString,QString> currencies;
  QMap<QString,qreal> currentRates;

  try
  {
    skipSignal = true;
    cbLeftCurrency->clear();
    cbRightCurrency->clear();

    currencies = module->currencies();
    currentRates = module->ratesAtDate();

    QMapIterator<QString,QString> currency(currencies);

    cbLeftCurrency->addItem("Euro (EUR)","EUR");
    cbRightCurrency->addItem("Euro (EUR)","EUR");
    while (currency.hasNext())
      if (currentRates.contains(currency.next().key()))
      {
        cbLeftCurrency->addItem(currency.value() + " (" + currency.key() + ")",
                                currency.key());
        cbRightCurrency->addItem(currency.value() + " (" + currency.key() + ")",
                                 currency.key());
      }

    cbLeftCurrency->setCurrentIndex(cbLeftCurrency->findData(module->fromCurrency()));
    cbRightCurrency->setCurrentIndex(cbRightCurrency->findData(module->toCurrency()));

    skipSignal = false;

    leLeftValue->setText("0.00");

    deExchangeDate->setDate(module->maximumExchangeDate());
  }
  catch(const QString &msg)
  {
    QErrorMessage(this).showMessage(msg);
  }
}

void MainWindow::downloadStarted()
{
  progressDlg = new QProgressDialog(this);
  progressDlg->setCancelButton(0);
  progressDlg->setAutoClose(false);
  progressDlg->setAttribute(Qt::WA_DeleteOnClose,true);
  progressDlg->setModal(true);
  progressDlg->setWindowTitle("Updating...");
  progressDlg->setWindowIcon(QIcon(":/images/update"));
}

void MainWindow::downloadFailed()
{
  progressDlg->close();
}

void MainWindow::downloadCompleted()
{
  progressDlg->close();

  centralWidget()->setEnabled(true);
  actionChart->setEnabled(true);
  reload();
}

void MainWindow::showStoreProgress(int step, int current, int total)
{
  switch (step)
  {
    case 0: progressDlg->setLabelText("Processing daily rates..."); break;
    case 1: progressDlg->setLabelText("Processing historical rates..."); break;
  }

  progressDlg->setMinimum(0);
  progressDlg->setMaximum(total);
  progressDlg->setValue(current);

  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
}

void MainWindow::showDownloadProgress(int step, int current, int total)
{
  switch (step)
  {
    case 0: progressDlg->setLabelText("Downloading daily rates..."); break;
    case 1: progressDlg->setLabelText("Downloading historical rates..."); break;
  }

  progressDlg->setMinimum(0);
  progressDlg->setMaximum(total);
  progressDlg->setValue(current);

  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
}

void MainWindow::recalculate()
{
  skipSignal = true;
  lblExchangeDirection->setPixmap(QPixmap((convertLeftToRight)? ":/images/to_right" :
                                                                ":/images/to_left"));
  deExchangeDate->setMaximumDate(qMin(module->maximumExchangeDate(cbLeftCurrency->itemData(cbLeftCurrency->currentIndex()).toString()),
                                      module->maximumExchangeDate(cbRightCurrency->itemData(cbRightCurrency->currentIndex()).toString())));
  deExchangeDate->setMinimumDate(qMax(module->minimumExchangeDate(cbLeftCurrency->itemData(cbLeftCurrency->currentIndex()).toString()),
                                      module->minimumExchangeDate(cbRightCurrency->itemData(cbRightCurrency->currentIndex()).toString())));
  if (convertLeftToRight)
    leRightValue->setText(QString("%1").arg(module->exchange(leLeftValue->text().toDouble(),
                                                             deExchangeDate->date()),0,'f',2));
  else
    leLeftValue->setText(QString("%1").arg(module->exchange(leRightValue->text().toDouble(),
                                                            deExchangeDate->date()),0,'f',2));
  skipSignal = false;
}
