#include <QtCore>
#include <QtGui>
#include <QtNetwork>
#include <QtXml>
#include <QtSql>

#include "qexchange.h"

QExchange::QExchange(int argc, char **argv) : QApplication(argc,argv)
{
  if (!QFileInfo(QDir::homePath() + "/.qexchange").exists())
    QDir(QDir::homePath()).mkdir(".qexchange");

  if (!QFileInfo(QDir::homePath() + "/.qexchange/db.sq3").exists())
  {
    QFile dbFile(QDir::homePath() + "/.qexchange/db.sq3");
    QDataStream dataStream(&dbFile);
    QResource dbResource(":/db/base");

    dbFile.open(QIODevice::WriteOnly);

    dataStream.writeRawData(reinterpret_cast<const char *> (dbResource.data()),dbResource.size());
    dbFile.close();
  }

  db = QSqlDatabase::addDatabase("QSQLITE");
  db.setDatabaseName(QDir::homePath() + "/.qexchange/db.sq3");
  db.open();

  db.exec("pragma synchronous = off");
  db.exec("pragma journal_mode = off");
  db.exec("pragma locking_mode = exclusive");

  connect(&ecbHttp,SIGNAL(downloadCompleted(bool)),this,SLOT(newRatesAvailable(bool)));
  connect(&ecbHttp,SIGNAL(downloadProgress(int,int,int)),
          this,SLOT(downloadStatus(int,int,int)));

  loadSettings();

  if (mainSettings["updateOnStart"].toBool())
    QTimer::singleShot(500,this,SLOT(update()));
}

QExchange::~QExchange()
{
  db.close();

  saveSettings();
}

void QExchange::loadSettings()
{
  QSettings settings("BeCrux","qexchange");

  if ((mainSettings["loadLastCurrencySetting"] = settings.value("loadLastCurrencySetting",false).toBool()).toBool())
  {
    mainSettings["lastFromCurrency"] = settings.value("lastFromCurrency","EUR").toString();
    mainSettings["lastToCurrency"] = settings.value("lastToCurrency","USD").toString();
  }
  else
  {
    mainSettings["lastFromCurrency"] = "EUR";
    mainSettings["lastToCurrency"] = "USD";
  }

  mainSettings["updateOnStart"] = settings.value("updateOnStart",false).toBool();

  settings.beginGroup("Network");
  mainSettings["useProxy"] = settings.value("useProxy",false).toBool();
  mainSettings["proxyType"] = settings.value("proxyType",QNetworkProxy::HttpProxy).toInt();
  mainSettings["proxyHost"] = settings.value("proxyHost","").toString();
  mainSettings["proxyPort"] = settings.value("proxyPort",8080).toInt();
  mainSettings["proxyUsername"] = settings.value("proxyUsername","").toString();
  mainSettings["proxyPassword"] = settings.value("proxyPassword","").toString();
  settings.endGroup();

  if (mainSettings["useProxy"].toBool())
    ecbHttp.setProxy(QNetworkProxy(static_cast<QNetworkProxy::ProxyType> (mainSettings["proxyType"].toInt()),
                                   mainSettings["proxyHost"].toString(),
                                   mainSettings["proxyPort"].toInt(),
                                   mainSettings["proxyUsername"].toString(),
                                   mainSettings["proxyPassword"].toString()));
}

void QExchange::saveSettings()
{
  QSettings settings("BeCrux","qexchange");

  settings.setValue("loadLastCurrencySetting",mainSettings.value("loadLastCurrencySetting",false));
  settings.setValue("lastFromCurrency",mainSettings.value("lastFromCurrency","EUR"));
  settings.setValue("lastToCurrency",mainSettings.value("lastToCurrency","USD"));
  settings.setValue("updateOnStart",mainSettings.value("updateOnStart",false));

  settings.beginGroup("Network");
  settings.setValue("useProxy",mainSettings.value("useProxy",false));
  settings.setValue("proxyType",mainSettings.value("proxyType",static_cast<int> (QNetworkProxy::HttpProxy)));
  settings.setValue("proxyHost",mainSettings.value("proxyHost",""));
  settings.setValue("proxyPort",mainSettings.value("proxyPort",8080));
  settings.setValue("proxyUsername",mainSettings.value("proxyUsername",""));
  settings.setValue("proxyPassword",mainSettings.value("proxyPassword",""));
  settings.endGroup();
}

void QExchange::setNewSettings(const QMap<QString,QVariant> &newSettings)
{
  mainSettings = newSettings;

  if (mainSettings["useProxy"].toBool())
    ecbHttp.setProxy(QNetworkProxy(static_cast<QNetworkProxy::ProxyType> (mainSettings["proxyType"].toInt()),
                                   mainSettings["proxyHost"].toString(),
                                   mainSettings["proxyPort"].toInt(),
                                   mainSettings["proxyUsername"].toString(),
                                   mainSettings["proxyPassword"].toString()));
  else
    ecbHttp.setProxy(QNetworkProxy(QNetworkProxy::NoProxy));
}

qreal QExchange::exchange(qreal value, const QDate &atDate)
{
  QString fromCurrency = mainSettings.value("lastFromCurrency","").toString();
  QString toCurrency = mainSettings.value("lastToCurrency","").toString();
  QSqlQuery query;
  int unixDate;
  QDate realDate = atDate;
  qreal fromRate, toRate;

  if (atDate.isValid())
    switch (atDate.dayOfWeek())
    {
      case Qt::Saturday: realDate = atDate.addDays(-1); break;
      case Qt::Sunday: realDate = atDate.addDays(-2); break;
      default: break;
    }

  unixDate = QDateTime((!realDate.isValid())? maximumExchangeDate() : realDate).toTime_t();

  if ((!fromCurrency.compare("EUR",Qt::CaseInsensitive)) || fromCurrency.isEmpty())
    fromRate = 1.0;
  else
  {
    query = db.exec(QString("select rate from rates where currency = '") + fromCurrency +
                            "' and unixdate = " + QString::number(unixDate));

    if (!query.next())
      return 0;

    fromRate = query.value(0).toDouble();
  }

  if ((!toCurrency.compare("EUR",Qt::CaseInsensitive)) || toCurrency.isEmpty())
    toRate = 1.0;
  else
  {
    query = db.exec(QString("select rate from rates where currency = '") + toCurrency +
                            "' and unixdate = " + QString::number(unixDate));

    if (!query.next())
      return 0;

    toRate = query.value(0).toDouble();
  }

  return (toRate * value / fromRate);
}

QMap<int,qreal> QExchange::rateHistory(const QString &from, const QString &to)
{
  QSqlQuery query;
  QMap<int,qreal> result;

  if (!from.compare("EUR",Qt::CaseInsensitive) && !to.compare("EUR",Qt::CaseInsensitive))
    query = db.exec(QString("select unixdate, 1.0, 1.0 from rates"));
  else if (!from.compare("EUR",Qt::CaseInsensitive))
    query = db.exec(QString("select unixdate, 1.0, rate from rates where currency = '") + to + "' order by unixdate");
  else if (!to.compare("EUR",Qt::CaseInsensitive))
    query = db.exec(QString("select unixdate, rate, 1.0 from rates where currency = '") + from + "' order by unixdate");
  else
    query = db.exec(QString("select a.unixdate, a.rate, b.rate from rates as a, rates as b ") +
                    "where a.unixdate = b.unixdate and a.currency = '" + from +
                    "' and b.currency = '" + to + "' order by a.unixdate");

  while (query.next())
    result[query.value(0).toInt()] = query.value(2).toDouble() / query.value(1).toDouble();

  return result;
}

void QExchange::setFromCurrency(const QString &currency)
{
  mainSettings["lastFromCurrency"] = currency;

  emit changed();
}

void QExchange::setToCurrency(const QString &currency)
{
  mainSettings["lastToCurrency"] = currency;

  emit changed();
}

QDate QExchange::maximumExchangeDate(const QString currency)
{
  QSqlQuery query;

  if (currency.isEmpty() || !currency.compare("EUR",Qt::CaseInsensitive))
    query = db.exec("select max(unixdate) from rates");
  else
    query = db.exec(QString("select max(unixdate) from rates where currency = '") + currency + "'");

  if (!query.next())
    throw QString("Invalid query in rates");

  return QDateTime::fromTime_t(query.value(0).toInt()).date();
}

QDate QExchange::minimumExchangeDate(const QString currency)
{
  QSqlQuery query;

  if (currency.isEmpty() || !currency.compare("EUR",Qt::CaseInsensitive))
    query = db.exec("select min(unixdate) from rates");
  else
    query = db.exec(QString("select min(unixdate) from rates where currency = '") + currency + "'");

  if (!query.next())
    throw QString("Invalid query in rates");

  return QDateTime::fromTime_t(query.value(0).toInt()).date();
}

bool QExchange::hasData(const QString currency)
{
  if (!currency.compare("EUR",Qt::CaseInsensitive))
    return true;
  else
  {
    QSqlQuery query;

    if (currency.isEmpty())
      query = db.exec("select count(*) from rates");
    else
      query = db.exec(QString("select count(*) from rates where currency = '") + currency + "'");

    if (!query.next())
      throw QString("Invalid query in rates");

    return query.value(0).toInt();
  }
}

QMap<QString,qreal> QExchange::ratesAtDate(const QDate &atDate)
{
  QMap<QString,qreal> result;
  QSqlQuery query;
  QDate realDate = atDate;
  int unixDate;

  if (atDate.isValid())
    switch (atDate.dayOfWeek())
    {
      case Qt::Saturday: realDate = atDate.addDays(-1); break;
      case Qt::Sunday: realDate = atDate.addDays(-2); break;
      default: break;
    }

  unixDate = QDateTime((!realDate.isValid())? maximumExchangeDate() : realDate).toTime_t();

  query = db.exec(QString("select currency, rate from rates where unixdate = " +
                  QString::number(unixDate)));

  while (query.next())
    result[query.value(0).toString()] = query.value(1).toDouble();

  return result;
}

QList<QDate> QExchange::ratesDates()
{
  QSqlQuery query;
  QList<QDate> result;

  query = db.exec("select distinct unixdate from rates order by unixdate");

  while (query.next())
    result.push_back(QDateTime::fromTime_t(query.value(0).toInt()).date());

  return result;
}

QMap<QString,QString> QExchange::currencies()
{
  QMap<QString,QString> result;
  QSqlQuery query;

  query = db.exec("select code, name from currencies");

  while (query.next())
    result[query.value(0).toString()] = query.value(1).toString();

  return result;
}

void QExchange::update()
{
  downloadStep = 0;
  emit startDownload();
  currentXmlFile = new QTemporaryFile;
  currentXmlFile->open();

  ecbHttp.download(QUrl("http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml"),currentXmlFile);
}

void QExchange::newRatesAvailable(bool error)
{
  QDate ratesDate;
  QDate currentDate;
  QByteArray response;

  if (!error)
  {
    currentXmlFile->close();
    currentXmlFile->open();

    QXmlStreamReader xmlResp(currentXmlFile);

    if (downloadStep == 0)
    {
      db.exec("delete from rates");
      db.exec("vacuum");
    }

    while (!xmlResp.atEnd())
      if ((xmlResp.readNext() == QXmlStreamReader::StartElement) &&
          !xmlResp.name().toString().compare("Cube",Qt::CaseInsensitive))
          if (!xmlResp.attributes().value("time").isEmpty())
            ratesDate = QDate::fromString(xmlResp.attributes().value("time").toString(),Qt::ISODate);
          else if (!xmlResp.attributes().value("currency").isEmpty() &&
                   !xmlResp.attributes().value("rate").isEmpty())
          {
            db.exec(QString("insert into rates values(") +
                    QString::number(QDateTime(ratesDate).toTime_t()) +
                    ",'" + xmlResp.attributes().value("currency").toString() + "'," +
                    xmlResp.attributes().value("rate").toString() + ")");

            emit storeInProgress(downloadStep,currentXmlFile->pos(),currentXmlFile->size());
          }

    switch (downloadStep++)
    {
      case 0:
        delete currentXmlFile;
        currentXmlFile = new QTemporaryFile;
        currentXmlFile->open();
        ecbHttp.download(QUrl("http://www.ecb.europa.eu/stats/eurofxref/eurofxref-hist.xml"),
                         currentXmlFile);
        break;

      case 1:
        delete currentXmlFile;
        emit downloadCompleted(); break;

      default:
        break;
    }
  }
  else
  {
    delete currentXmlFile;
    emit downloadFailed();
  }
}

void QExchange::downloadStatus(int /* tries */, int done, int total)
{
  emit downloadInProgress(downloadStep,done,total);
}
